@@ -41,7 +41,8 @@ const R_CallMethodDef callMethods[] = {
 void R_init_CRF(DllInfo *info)
 {
   R_registerRoutines(info, NULL, callMethods, NULL, NULL);
-  R_useDynamicSymbols(info, TRUE);
+  R_useDynamicSymbols(info, FALSE);
+  R_forceSymbols(info, TRUE);
 }
 
 CRF::CRF()
